/*==========================================================
 * Program : pdu.h                         Project : smslink
 * Author  : Philippe Andersson.
 * Date    : 18/10/01
 * Version : 0.05b
 * Notice  : (c) Les Ateliers du Heron, 1998 for Scitex Europe, S.A.
 * Comment : PDU-mode include file for SMSLink server.
 *
 * Modification History :
 * - 0.01a (07/04/00) : Initial release.
 * - 0.02a (13/04/00) : Added the min() macro (moved from server.y).
 * ++++ Switch to Beta ++++
 * - 0.03b (21/11/00) : Added the decode_pdu() function definition.
 * - 0.04b (23/11/00) : Adapted the decode_pdu() function def.
 *   to include the cap_matrix field.
 * - 0.05b (18/10/01) : Exported ascii7topacked8() and
 *   ascii7topacked8() for external use.
 *========================================================*/

#ifndef _PDU_H
#define _PDU_H

#define min(a,b)   ( ( (a) < (b) ) ? (a) : (b) )

/* TPDU Flags (see GSM 03.40) */
#define TP_MTI_SMS_DELIVER	 0	/* Message type : DELIVER */
#define TP_MTI_SMS_SUBMIT	 1	/* Message type : SUBMIT */
#define TP_MMS_NO_MORE_MSG	 4	/* No more msg waiting */
#define TP_MMS_MORE_MSG	 	 0	/* More msg waiting */
#define TP_VPF_NOT_PRESENT	 0	/* No validity period field */
#define TP_VPF_RELATIVE		16	/* Relative VP field */
#define TP_VPF_ABSOLUTE		24	/* Absolute VP field */
#define TP_SRI_SR_NOT_REQD	 0	/* No status rep. requested */
#define TP_SRI_SR_REQD		32	/* Status report requested */

/* PID */
#define PID_SMS			 0	/* Protocol ID : SMS */

/* DCS (Data Coding Scheme) */
#define DCS_PCCP437		 0	/* PCCP437 8bit -> 7bit */

/*==========================================================
 * Structure Declarations
 *========================================================*/

/*==========================================================
 * Function Declarations
 *========================================================*/
/* --- in "pdu.c" --- */
int ascii7topacked8 (char *, char *);
int packed8toascii7 (char *, char *);
char *encode_pdu (struct symbols *);
int decode_pdu (char *, struct mbox_item *, cap_matrix);

#endif                                  /* #ifndef _PDU_H */

/*==========================================================
 * EOF : pdu.h
 *===================*/
